from __future__ import annotations
from typing import Dict, Tuple, TYPE_CHECKING, Union, Optional, List

import pygame
from pygame.rect import Rect
from pygame.surface import Surface

if TYPE_CHECKING:
    from .Theme import Theme


class Tileset:
    def __init__(self, theme: Theme, tileSize: Tuple[int, int], imageFile: str):
        self.__theme = theme
        self.__tileSize = tileSize
        self.__imageFile = imageFile
        self.__surface: Optional[Surface] = None
        self.__tilesRect: Dict[Union[int, str], Rect] = {}

    @property
    def tileSize(self) -> Tuple[int, int]:
        return self.__tileSize

    @property
    def surface(self) -> Surface:
        if self.__surface is None:
            self.__surface = self.__theme.getSurface(self.__imageFile)
        return self.__surface

    def addTile(self, value: Union[int, str], coords: Tuple[int, int]):
        self.__tilesRect[value] = Rect(
                coords[0] * self.__tileSize[0],
                coords[1] * self.__tileSize[1],
                self.__tileSize[0], self.__tileSize[1]
            )

    def getTileRect(self, value: Union[int, str]) -> Rect:
        if value not in self.__tilesRect:
            raise ValueError(f"No {value} in tileset {self.__imageFile}")
        return self.__tilesRect[value]

    def getTile(self, value: Union[int, str]) -> Surface:
        rect = self.getTileRect(value)
        surface = Surface(rect.size, flags=pygame.SRCALPHA)
        surface.blit(self.surface, (0, 0), rect)
        return surface

    def blitTile(self, surface: Surface, dest: Tuple[int, int], value: Union[int, str]):
        rect = self.getTileRect(value)
        surface.blit(surface, dest, rect)

    def getTilesId(self) -> List[Union[int, str]]:
        return list(self.__tilesRect.keys())

    def getTilesRect(self) -> Dict[Union[int, str], Rect]:
        return self.__tilesRect

